      PROGRAM AUSTGROW
C
C	This program implements a simple model of austenite growth
C	in hypoeutectoid steels that have been rolled and present
C	a ferritic pearlitic structure, with oriented banding.
C
C
C
C
C
C
C       Written by Daniel Gaude-Fugarolas. Sept.-March 2001
C
C	...	parameter declaration	... 
C
      INTEGER  I, J, TFACTOR
      INTEGER IIU1, IIU2, IOU, ERR, INIOUT, CTRL
      DOUBLE PRECISION  R, ZEROKELVIN, BOLTZ, HH
      DOUBLE PRECISION LALPHA, LHYPO, LEUT, LPEARL, LCOL
      DOUBLE PRECISION TEMP, TEMPK, TIME, INCTIME, INCTEMP
      DOUBLE PRECISION NEXTTIME, NEXTTEMP
      DOUBLE PRECISION VINT,  BIGR, DIFFCM, DIFFM, WDIFF
      DOUBLE PRECISION RINT1, RINT2, RINT3
      DOUBLE PRECISION VAUS, VPER, VFFR, VPFR, VPCM
      DOUBLE PRECISION CAUSCEM, CAUSFER, CFERINF, CAUS, COMP(1:8)
      DOUBLE PRECISION AE1, XBAR, T20, T10
      DOUBLE PRECISION EXTRATIME
      DOUBLE PRECISION EQVAUS, EVALEQVAUS, C(1:10)
      DOUBLE PRECISION IZERO, Q
C
C
C mb...Variables for the mass balance of carbon (DIFFUSED, INTERFACE)...
      DOUBLE PRECISION MBD, MBI, TMB, MBF
C
C
      PARAMETER (IIU1 = 1)
      PARAMETER (IIU2 = 2)
      PARAMETER (IOU = 4)
C
      DATA R,ZEROKELVIN,BOLTZ,HH/8.314,273.0,1.38E-23,6.624E-34/
      ERR = 0
      INIOUT = 0
      CTRL = 0
      TIME = 0.0
      TEMP = -ZEROKELVIN
      RINT1 = 0.0
      RINT2 = 0.0
      RINT3 = 0.0
      TFACTOR = 1000
      EXTRATIME = 0.0
C

C ... Mole % carbon concentration of eutectoid austenite...
      CAUS = 3.61D-2
      J = 0
C
C     PRINT *,'...Get input data from files...'
      PRINT *,' ...and opens output file ...'
C
      CALL GETDAT (IIU1, COMP, LALPHA, LHYPO, LEUT, LPEARL, 
     &  LCOL, AE1, IZERO, Q, ERR)
C
      VAUS = 0.0
      VPER = LPEARL/LHYPO
      VFFR = LALPHA/LHYPO
      VPFR = 0.0
      VPCM = 0.0
C
C  ...Opens output file and prepares to add results in there...
C
C
C   ...gets composition in right order for Map_Steel_Omega...
C
      DO 13 , I=1, 8
         C(I) = COMP(I)
 13   CONTINUE
      C(4) = COMP(5)
      C(5) = COMP(6)
      C(6) = COMP(4)
C 
      PRINT *,'...Opening files ...'
C
C
      CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
      INIOUT = 1
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C

      CTRL = 1
C
C
C ...the steel is heated until Ae1 is reached and the reaction starts...
C
      PRINT *,'... Heating to reaction temperature  ...'
C
      OPEN (UNIT = IIU2, FILE = "temperature", STATUS = 'OLD')
C
C
      CALL  MAP_STEEL_OMEGA(C,WDIFF,XBAR,T10,T20,J)
C
C     ...In this subroutine the composition COMP(1:8) has been
C        to atomic %  ...
C
C
      READ (IIU2, *) NEXTTIME, NEXTTEMP
  10  IF(NEXTTIME.LT.(0.0)) GOTO 99
      IF (NEXTTEMP.LE.AE1) THEN
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C
         CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
         TIME = NEXTTIME
         TEMP = NEXTTEMP
         READ (IIU2, *) NEXTTIME, NEXTTEMP
         GOTO 10
      ENDIF
C
C ...Ae1 has been reached and the reaction begins...
C
      PRINT*,'...The austenitisation of pearlite begins...'
C
      BIGR = LEUT
      RINT1 = 1.0E-8
C   ...  Here instantaneous nucleation is being considered ...
C
C   ... not always the time intervals in the input file will be
C        coincident with the transformation start temperatures...
C
  70  IF (NEXTTEMP.GT.AE1.AND.TEMP.LE.AE1) THEN
         INCTEMP = NEXTTEMP - AE1
         INCTIME = ((NEXTTEMP-AE1)/(NEXTTEMP-TEMP))*(NEXTTIME-TIME)
C
      ELSE IF((NEXTTEMP.LT.AE1).AND.(TEMP.GE.AE1)) THEN
         INCTEMP =  AE1 - TEMP
         INCTIME = ((AE1 - TEMP)/(NEXTTEMP-TEMP))*(NEXTTIME - TIME)
C
      ELSE IF((NEXTTEMP.GT.AE1).AND.(TEMP.GT.AE1)) THEN
         INCTEMP = NEXTTEMP - TEMP
         INCTIME = NEXTTIME - TIME
      ENDIF
C
C
      IF((NEXTTEMP.LE.AE1).AND.(TEMP.LE.AE1)) GOTO 10
C
C
      IF(RINT1.GE.BIGR) GOTO 20
C
C
      INCTIME = INCTIME / TFACTOR
      INCTEMP = INCTEMP / TFACTOR
C
      DO 25, I=1, TFACTOR
         TEMPK = TEMP +I*INCTEMP + ZEROKELVIN
C
C
          CALL GETCOMP(IIU3,TEMPK,CAUSCEM,CAUSFER,CFERINF,COMP)
C
          CALL MAP_STEEL_DIFFUS(DIFFCM,XBAR,TEMPK,WDIFF,R,BOLTZ,HH)
C                                                         
          DIFFM=DIFFCM/10000.0
C
          VINT=(DIFFM/(RINT1))*DABS((CAUSCEM-CAUSFER)/
     &         (CAUSFER-CFERINF))
C
C
          RINT1 = RINT1 + VINT*INCTIME
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, TEMPK)
C
C
C
C
  25  CONTINUE
C
C   ...Check for the extra time that could be used in later stages
C       after the first transformation has reached to an end... 
C
      IF(RINT1.GE.BIGR)THEN
         EXTRATIME = ABS(((RINT1-VINT*INCTIME) - BIGR)/VINT)
         RINT1 = BIGR
      ENDIF
C
C
C
       CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
C
      TIME = NEXTTIME
      TEMP = NEXTTEMP
C
      READ (IIU2, *) NEXTTIME, NEXTTEMP
      IF(NEXTTIME.LT.(0.0)) GOTO 99
C
C
      GOTO 70
C
C
C*****
  30  IF(NEXTTIME.LT.(0.0)) GOTO 99
      IF (NEXTTEMP.LE.AE1) THEN
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C
         CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
         TIME = NEXTTIME
         TEMP = NEXTTEMP
         READ (IIU2, *) NEXTTIME, NEXTTEMP
         GOTO 30
      ENDIF
C
C ...Steady growth of austenite begins...
C
 20   PRINT*,'...Steady growth of austenite in pearlite colonies...'
C
      BIGR = LCOL
      RINT3 = RINT1
C
C   ... not always the time intervals in the input file will be
C        coincident with the transformation start temperatures...
C
  75  IF (NEXTTEMP.GT.AE1.AND.TEMP.LE.AE1) THEN
         INCTEMP = NEXTTEMP - AE1
         INCTIME = ((NEXTTEMP-AE1)/(NEXTTEMP-TEMP))*(NEXTTIME-TIME)
C
      ELSE IF((NEXTTEMP.LT.AE1).AND.(TEMP.GE.AE1)) THEN
         INCTEMP =  AE1 - TEMP
         INCTIME = ((AE1 - TEMP)/(NEXTTEMP-TEMP))*(NEXTTIME - TIME)
C
      ELSE IF((NEXTTEMP.GT.AE1).AND.(TEMP.GT.AE1)) THEN
         INCTEMP = NEXTTEMP - TEMP
         INCTIME = NEXTTIME - TIME
      ENDIF
C
C
      IF((NEXTTEMP.LE.AE1).AND.(TEMP.LE.AE1)) GOTO 30
C
C*
      IF(VPER.LT.(1.0D-4)) GOTO 60
C
C
      INCTIME = INCTIME / TFACTOR
      INCTEMP = INCTEMP / TFACTOR
C
      DO 35, I=1, TFACTOR
         TEMPK = TEMP +I*INCTEMP + ZEROKELVIN
C
C
        EQVAUS = EVALEQVAUS(COMP, TEMP, AE1)
C
         IF(VAUS.GE.EQVAUS) THEN
C
            VINT = 0.0
            VAVE = 0.0
C
         ELSE IF (VAUS.LT.EQVAUS) THEN
C
            CALL GETCOMP(IIU3,TEMPK,CAUSCEM,CAUSFER,CFERINF,COMP)
C
            CALL MAP_STEEL_DIFFUS(DIFFCM,XBAR,TEMPK,WDIFF,R,BOLTZ,HH)
C                                                         
            DIFFM=DIFFCM/10000.0
C
            VINT=(DIFFM/(RINT1))*DABS((CAUSCEM-CAUSFER)/
     &         (CAUSFER-CFERINF))
C
            VAVE = (DIFFM/RINT1)*(LOG(RINT1)-LOG(1.0D-8))*
     &         DABS((CAUSCEM-CAUSFER)/(CAUSFER-CFERINF))
C
         ENDIF
C
          RINT3 = RINT3 + VAVE*INCTIME
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, TEMPK)
C
C
C
  35  CONTINUE
C
C   ...Check for the extra time that could be used in later stages
C       after the first transformation has reached to an end... 
C
      IF(RINT3.GE.BIGR)THEN
C
         IF(VAVE.LT.1.0D-8)THEN
C
            EXTRATIME = 0.0
C
         ELSE IF(VAVE.GE.(1.0D-8))THEN
C
            EXTRATIME = ABS(((RINT3-VAVE*INCTIME) - BIGR)/VAVE)
            RINT3 = BIGR
C
         ENDIF
C
      ENDIF
C
C
C
       CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
C
      TIME = NEXTTIME
      TEMP = NEXTTEMP
C
      READ (IIU2, *) NEXTTIME, NEXTTEMP
      IF(NEXTTIME.LT.(0.0)) GOTO 99
C
C
      GOTO 75
C
C*****
C ... Austenitisation of ferrite ...
C
  60  PRINT *,'... Austenitisation of ferrite ...'
C
C
      BIGR =  LALPHA
      RINT2 = 1.0E-8
C   ...  New grains of austenite grow from previous austenite ...
C
C   ... not always the time intervals in the input file will be
C        coincident with the transformation start temperatures...
C
 170  IF (NEXTTEMP.GT.AE1.AND.TEMP.LE.AE1) THEN
         INCTEMP = NEXTTEMP - AE1
         INCTIME = ((NEXTTEMP-AE1)/(NEXTTEMP-TEMP))*(NEXTTIME-TIME)
C
      ELSE IF((NEXTTEMP.LT.AE1).AND.(TEMP.GE.AE1)) THEN
         INCTEMP =  AE1 - TEMP 
         INCTIME = ((AE1 - TEMP)/(NEXTTEMP-TEMP))*(NEXTTIME - TIME)
      ENDIF
C
C
C
      IF((NEXTTEMP.LE.AE1).AND.(TEMP.LE.AE1)) THEN
  110       IF(NEXTTIME.LT.(0.0)) GOTO 99
         IF (TEMP.LE.AE1) THEN
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C
C
           CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
           TIME = NEXTTIME
           TEMP = NEXTTEMP
           READ (IIU2, *) NEXTTIME, NEXTTEMP
           GOTO 110
         ENDIF
      ENDIF
C
C
      IF(RINT2.GE.BIGR) GOTO 120
C
C
      INCTIME = INCTIME + EXTRATIME
      INCTIME = INCTIME / TFACTOR
      INCTEMP = INCTEMP / TFACTOR
      EXTRATIME = 0.0
C
      DO 125, I=1, TFACTOR
         TEMPK = TEMP +I*INCTEMP + ZEROKELVIN
C
         EQVAUS = EVALEQVAUS(COMP, TEMP, AE1)
C
         IF(VAUS.GE.EQVAUS) THEN
C
            VINT = 0.0
C
         ELSE IF(VAUS.LT.EQVAUS) THEN
C
            CALL GETCOMP(IIU3,TEMPK,CAUSCEM,CAUSFER,CFERINF,COMP)
C
            CALL MAP_STEEL_DIFFUS(DIFFCM,XBAR,TEMPK,WDIFF,R,BOLTZ,HH)
C
            DIFFM = DIFFCM/10000.0
C                                                         
            VINT=(DIFFM/RINT2)*DABS((CAUS-CAUSFER)/
     &         (CAUSFER-CFERINF))
C
         ENDIF
C
         RINT2 = RINT2 + VINT*INCTIME
C
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, TEMPK)
C
C
C
C mb  ... Evaluation and print of the mass balance ...
C    ... TMB shows the excess of diffused atoms ...
C 
      MBI = (CAUSFER -CFERINF)*(VINT*INCTIME)
      MBD =  DIFFM*((CAUS - CAUSFER)/RINT2)*INCTIME
      TMB = MBD - MBI
C*      PRINT *, 'C bal. intf.:', MBD, MBI, (MBD/MBI)
C endmb
C
C
  125  CONTINUE
C
C   ...Check for the extra time that could be used in later stages
C       after the first transformation has reached to an end... 
C
      IF(RINT2.GE.BIGR)THEN
         EXTRATIME = ((RINT2-VINT*INCTIME) - BIGR)/VINT
         RINT2 = BIGR
         CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
      ENDIF
C
C
      CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
C
      TIME = NEXTTIME
      TEMP = NEXTTEMP
C
      READ (IIU2, *) NEXTTIME, NEXTTEMP
      IF(NEXTTIME.LT.(0.0)) GOTO 99
C
      INCTIME = ABS(NEXTTIME - TIME)
      INCTEMP = NEXTTEMP - TEMP
C
      GOTO 170
C
  120 PRINT *,'Austenitisation of ferrite completed'
C 
  130 IF(NEXTTIME.LT.(0.0)) GOTO 99
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C
C
      CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
C
      TIME = NEXTTIME
      TEMP = NEXTTEMP
      READ (IIU2, *) NEXTTIME, NEXTTEMP
      GOTO 130
C
  90  PRINT *,'The austenitisation temperature has not been reached'  
C
C
 99   PRINT *,'End of Time - Temperature data'
C
      CALL VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, (TEMP+ZEROKELVIN))
C
      CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
C
      print *,'.......Final Vaus=  ', VAUS
C*      INIOUT = 2
      CALL OUTDAT2(IOU,TIME,TEMP,VAUS,VPER,VFFR,VPFR,VPCM,INIOUT)
      CLOSE(IIU1)
      CLOSE(IIU2)
      STOP
      END
C
C***********************************************************************
C    *******************************************************************
C
C    Subroutine GETDAT to read from a file the parameters to run the
C    model
C
      SUBROUTINE GETDAT  (IIU, COMP, LALPHA, LHYPO, LEUT, LPEARL,
     *    LCOL, AE1, IZERO, Q, ERR)
C
C
      INTEGER IIU, I, ERR 
      DOUBLE PRECISION COMP(8), LALPHA, LHYPO, LEUT, LPEARL, LCOL
      DOUBLE PRECISION AE1, IZERO, Q
      CHARACTER*55 FILE1
C
C	... ask for file name, open it, get data, close it ...
C
C*      WRITE(*,*) 'Input file name?'
C*      READ (*,*) FILE1
C*      OPEN (UNIT = IIU, FILE = FILE1, STATUS = 'OLD')
      PRINT *,'using data from "inputfile3"'
C*
      OPEN (UNIT = IIU, FILE = 'inputfile3', STATUS = 'OLD')
C
      READ (IIU, *) (COMP(I), I=1, 7)
      READ (IIU, *) LALPHA, LHYPO, LEUT, LPEARL, LCOL
      READ (IIU, *) AE1
      READ (IIU, *) IZERO, Q
C
      CLOSE (IIU)
C
      RETURN
      END
C
C     ******************************************************************
C     ******************************************************************
C     ******************************************************************
C
C
      SUBROUTINE OUTDAT2(IOU,  TIME, TEMP, VAUS, VPER, VFFR,
     & VPFR, VPCM, INIOUT)
C
C
C     Output of results. 
C     With iniout = 0, asks for file name and opens it
C     With iniout = 1, adds a new line to the file 
C     (time and temperatures)
C     with iniout = 2, closes file
C
C
C
      INTEGER IOU, INIOUT
      DOUBLE PRECISION TIME, TEMP, VTOT 
      DOUBLE PRECISION  VAUS, VPER, VFFR, VPFR, VPCM
      CHARACTER*55 FILE2
C
C   ...If iniout = 0, only asks for file name and opens it...
C
      IF(INIOUT.EQ.0) THEN 
C*         WRITE(*,*) 'Output file name?'
C*         READ (*,*) FILE2
C*         OPEN (UNIT = IOU, FILE = FILE2, STATUS = 'UNKNOWN')
         PRINT *,'Output data stored in "outputfile2" '
         OPEN (UNIT = IOU, FILE = 'outputfile2', STATUS = 'UNKNOWN')
         WRITE(IOU, 1001)
C
C   ...if iniout = 1 adds data to file...
C
      ELSE IF(INIOUT.EQ.1) THEN
         VTOT = VAUS + VPER + VFFR
         WRITE(IOU, 1003) TIME, TEMP, VAUS, VPER, VFFR, VPFR, VPCM
     &  ,VTOT
C
C   ...if iniout = 2, closes file...
C
      ELSE IF(INIOUT.EQ.2) THEN
         CLOSE (IOU)
      END IF
C
      RETURN
C   ...Formats...
C
 1001 FORMAT(1H , 'Time /s', 2X, 'Temp./C',2X,'Vaus',3X 'Vpearl',
     &     3X, 'Vfer',4X, 'Vpfr', 4X, 'Vpcm', 5X, 'Total', 5X)
 1003 FORMAT(1H ,F6.2,3X,F6.1,2X,F6.4,2X,F6.4,2X,F6.4,2X,F6.4,
     &  2X,F6.4, 4X, F6.4)
C
      END
C
C
C **********************************************************************
C **********************************************************************
C
C
C
C
C
      SUBROUTINE GETCOMP(IIU3,TK,CAUSCEM,CAUSFER,CFERINF,COMP)
C
C
C  ...  Subroutine to calculate or read from a file the 
C       equilibrium or paraequilibrium compositions at 
C       the interfaces and in the bulk ferrite and cementite ...
C
C
C  ...  At the  moment it calculates them from equations 
C       derived from results by Akbay et al. [bg] pp. 1473  ...
C  ...    for Fe-C steel   ...
C
C
      INTEGER IIU3
      DOUBLE PRECISION TK, CAUSCEM, CAUSFER, CFERINF
      DOUBLE PRECISION COMP(1:8)
      DOUBLE PRECISION TC
C
C
      TC = TK - 273
C
      CFERINF = 0.0
      CAUSFER = 1.00124020384 - 0.14731818505*DLOG(TC)
      IF (CAUSFER.LE.0.0011) THEN
          CAUSFER = 0.0011
      ENDIF
      CAUSCEM = 2.60127E-8 *TC**2.14054207323
C
      RETURN
C
      END
C
C***********************************************************************
C***********************************************************************
C
      SUBROUTINE VOLFRAC(COMP, LALPHA, LHYPO, LEUT, LPEARL, LCOL, 
     &    RINT1, RINT2, RINT3, VAUS, VPER, VFFR, VPFR, VPCM, CTRL,
     &    IZERO, Q, TK)
C
C
C  ...Subroutine that calculates the volume fraction of 
C     each phase from the original phase content and the
C     positions of the austenite interfaces.
C
C
C  ...    Variable declaration   ...
C
      INTEGER I, J, N, CTRL
      DOUBLE PRECISION LALPHA, LHYPO, LEUT, LPEARL, LCOL
      DOUBLE PRECISION RINT1, RINT2, RINT3, VTOT, VAUS
      DOUBLE PRECISION VPER, VFFR, VPFR, VPCM
      DOUBLE PRECISION RCOL, ATOT, ATRF, VTRFE, VTRF, RJ
      DOUBLE PRECISION THETA1, THETA2
      DOUBLE PRECISION PI
      DOUBLE PRECISION IZERO, Q, EVALI, IT, TK
C
      DOUBLE PRECISION COMP(1:8)
C
      SAVE ATOT, RCOL, N, VTRF, IT
C
      DATA PI /3.1415926535898/
C
C  ...   First time this routine  is used, total area
C        of a pearlite colony is calculated, etc.     ...
C
         ATRF = 0.0
         VTRFE = 0.0
C
C 
      IF(CTRL.EQ.0) THEN
C
         RCOL = LCOL / 2.0
         ATOT = 0.0
         VTRF = 0.0
C
C   ... Aprox 2N+1 is the number of ferrite/cementite pairs ...
C     
         N = INT(RCOL/LEUT)
C
         DO 53 I = (-N), N, 1
C
            ATOT=ATOT+PI*DABS((RCOL*DSIN(DACOS(LEUT*I/RCOL))))**2
C
  53     CONTINUE
C
         GOTO 89
C
      ENDIF
C
C  ...  Initial austenite growth in pearlite colonies  ...
C
      IF((RINT1.GT.(1.0E-8)).AND.(RINT3.LT.(1.1E-8))) THEN
C
         IT = MAX(IT, EVALI(IZERO, Q, TK, N, RCOL, COMP))
C
C
C**         VTRFE = IT*(2.0/3.0)*PI*RINT1**3
        VTRFE = IT*(RINT1/LCOL)**3
C
         VTRF = MAX(VTRF, (1.0 - EXP(-VTRFE)))
C
      ENDIF
C
C  ...  When austenite grows at steady speed in pearlite
C       its volume fraction in pearlite is calculated here  ...
C
C
      IF(RINT3.GT.(1.1E-8))THEN
C
         IT = MAX(IT, EVALI(IZERO, Q, TK, N, RCOL, COMP))
C
C
         DO 54 J=(-N), N, 1
C
            RJ = RCOL*DSIN(DACOS(LEUT*J/RCOL))
C
            IF(RINT3.GE.(2.0*RJ)) THEN
C
C
               ATRF = PI*RJ**2.0
C
            ELSE IF(RINT3.LT.(2.0*RJ)) THEN
C
               THETA2 = 2.0*DACOS(RINT3/(2.0*RJ))
C
               THETA1 = 2.0 *(PI - THETA2)
C
               ATRF = 0.5*((RJ**2)*(THETA1-DSIN(THETA1)) +
     &                  (RINT3**2)*(THETA2 - DSIN(THETA2)))
C
           ENDIF
C
         VTRFE = VTRFE + IT*ATRF/(ATOT*(2.0*N+1.0))
C
  54     CONTINUE
C
C  ...  Correction by extended volume method  ...
C
         VTRF = MAX(VTRF, (1.0 - EXP(-VTRFE)))
C 
C  ...   Correction to avoid assimptotic growth delay  ...
C
         IF(VTRF.GE.(0.99)) THEN
C
            VTRF = 1.0
            RINT3 = LCOL
C
         ENDIF
C
      ENDIF
C
C  ...    Volume fraction of pearlite    ...
C
      VPER = (LPEARL*(1.0 - VTRF))/LHYPO
C
C  ...    Volume fraction of pearlitic ferrite WRONG!!   ...
C
C**      VPFR = 0.885 * VPER
C
C  ...    Volume fraction of pearlitic cementite WRONG!!   ...
C
C**      VPCM = 0.115 * VPER
C
C  ...    Volume fraction of free ferrite    ...
C
      VFFR = (LALPHA - RINT2)/LHYPO
C
C  ...    Volume fraction of austenite    ...
C
      VAUS = ((LPEARL*VTRF) + RINT2)/LHYPO
C
C  ...    Total volume fraction of phases (=1.0?)    ...
C
      VTOT = VPER + VFFR + VAUS
      VTOT = DABS(VTOT - 1.0)
      IF (VTOT.GT.(0.001)) THEN
         PRINT *,'Error in total volume fractions ', (VTOT + 1.0)
      END IF
C
  89  RETURN
      END
C
C
C***********************************************************************
C**********************************************************************
C
      DOUBLE PRECISION FUNCTION EVALEQVAUS(COMP, TEMP, AE1)
C
C
C  ... Function that takes the value of the austenite equilibrium 
C      volume fraction between Ae1 and Ae3, 0.0 below Ae1 and 1.0 
C      above Ae3. Function fitted from data calculated using MTDATA ...
C
C  ...    Composition of alloy UC1-67  ...
C
C
C  ...    Variable declaration    ....
C
      DOUBLE PRECISION  COMP(1:8), TEMP, AE1
      DOUBLE PRECISION XC, EVALEQVAUS, T
C
C  ...   Calculations   ...
C
      XC = COMP(1)
      EVALEQVAUS = 0.0
      T = TEMP + 273
C
      IF(TEMP.GE.AE1) THEN
C
C   ... This is at the moment calculated only for UC1-67 ...
C
         EVALEQVAUS = (XC / (1.831D-5 * T**2 -4.394D-2 * T +26.34))
C
         IF(EVALEQVAUS.GT.(1.0)) THEN
C
            EVALEQVAUS = 1.0
C
         ENDIF
C
      ENDIF
C
      RETURN
C
      END
C
C
C
C***********************************************************************
C************************************************************************
C************************************************************************
C
C  ...Function to evaluate the dependence with temperature in nuclei
C      density when transforming from pearlite to austenite ...
C
      DOUBLE PRECISION FUNCTION EVALI(I0, C2, TK, N, Rcol, COMP)
C
C  ...  Variable declaration   ...
C
      DOUBLE PRECISION I0, C2, TK, N, Rcol, GASC, GVOL, COMP(1:8)
      DOUBLE PRECISION EVALI, R, H, K, PI, GAMMA, NZERO,EVGVOL
      DOUBLE PRECISION  M3MOL
C
      DATA  M3MOL / 7.105E-6/
      DATA R, PI, K, H, GAMMA, NZERO/8.314,3.141592653,1.38E-23,
     &   6.624E-34,0.025, 6.023E23/
C
C  ...  Evaluation   ...
C
      GVOL = EVGVOL(TK, COMP)
C
      GASC = C2*PI*(GAMMA**3)/(GVOL**2)
C
      GMOL = GASC*M3MOL/((32/3)*PI*((GAMMA/GVOL)**3))
C
      EVALI = I0*(3/Rcol)*K*TK/H*EXP(-(GMOL +270000)/(R*TK))
C
      RETURN
C
      END
C
C************************************************************************
C************************************************************************
C************************************************************************
C
      DOUBLE PRECISION FUNCTION EVGVOL(TK, COMP)
C
C   ...  Calculates the difference in gibbs energy between ferrite&
C        cementite (taking into account composition) and austenite per
C        unit volume using a neural network which weights have been 
C        determined from data calculated using MTDATA and ...
C
C             ...Daniel Gaude-Fugarolas--March 2001 ...
C
      DOUBLE PRECISION TK, COMP(1:8)
      DOUBLE PRECISION  RHO
      DOUBLE PRECISION TEMP
      INTEGER CTRL, MAXVARS, MAXHU, VARS, HU, INW, INM, I, J, K
      PARAMETER (MAXHU=20)
      PARAMETER (MAXVARS=20)
      PARAMETER (INW=3)
      PARAMETER (INM=5)
      DOUBLE PRECISION BIAS(MAXHU), WEIGHTS(MAXHU, MAXVARS), 
     &         HUBIAS, HUWEIGHTS(MAXHU),MINMAX(2,MAXVARS+1)
      DOUBLE PRECISION SUMHU, SUMVARS, XNN(MAXVARS)
      CHARACTER*12 WEINAME, FMINMAX
      DATA  RHO / 7800.0 /
C
      SAVE  BIAS, WEIGHTS, CTRL, XNN, MINMAX,HUBIAS, HUWEIGHTS
C
      SUMHU = 0.0
C
      IF(CTRL.NE.13) THEN
C*         WRITE(*,*)' .Information about the neural network.'
C*         WRITE(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
C*         WRITE(*,*)'Input weight file name'
C*         READ (*,*) WEINAME
C*         WRITE(*,*)'Input MINIMAX file name'
C*         READ (*,*) FMINMAX
C*         WRITE(*,*)'Input number of variables'
C*         READ (*,*) VARS
C*         WRITE(*,*)'Input number of hidden units'
C*         READ (*,*) HU
        hu =13
        vars =8
        WEINAME = 'weightfile'
        FMINMAX= 'minmaxfile'
C
         OPEN(UNIT=INW, FILE=WEINAME, STATUS='OLD')
C
         DO 67 I=1, HU
            READ(INW,*, END=7) BIAS(I)
            DO 77 J=1, VARS
               READ(INW,*, END=7) WEIGHTS(I, J)
  77        CONTINUE
  67     CONTINUE
C
         READ(INW,*, END=7) HUBIAS
         DO 97 J=1, HU
            READ(INW,*, END=7) HUWEIGHTS(J)
  97     CONTINUE
C 
C 
         CLOSE(INW)
C
         OPEN(UNIT=INM, FILE=FMINMAX, STATUS='OLD')
C        
         DO 87 K=1, VARS+1
            READ(INM,*) MINMAX(1,K),MINMAX(2,K)
C
C  ... The difference is stored instead of the maximum ...
C
            MINMAX(2,K) = MINMAX(2,K) - MINMAX(1,K)
C
  87     CONTINUE
C
         CLOSE(INM)
C
C  ... And the vector of compositions ...
C
C  ... Fe  ...
C
         TEMP = 100.0
C 
         DO 32 I=1, 6
C
           TEMP = TEMP - COMP(I)
C 
  32     CONTINUE 
C
          XNN(1) = (((TEMP/100.0) - MINMAX(1, 1)) / 
     &          MINMAX(2,1)) - 0.5 
C
C       
         DO 37 J=2, 7
              XNN(J) = (((COMP(J-1)/100.0) - MINMAX(1, J)) / 
     &            MINMAX(2,J)) - 0.5          
  37     CONTINUE 

C
C  ...temperature is left outside the startup loop ...
C
         CTRL = 13
      ENDIF
C
C  ... Once the files have been read, they'll be kept for the rest of
C       the use of the program.
C
C  ... Now the calculations of free energy can be done using the NN ...
C
C  ...  Temperature  ...
C
      XNN(8) = ((TK - MINMAX(1, 8)) / MINMAX(2,8)) - 0.5
C
C  ...  Start of the calculations  ...
C
      SUMHU = 0.0
C
      DO 38 I=1, HU  
C
         SUMVARS = 0.0
C
         DO 48 J=1, VARS
C
            SUMVARS = SUMVARS + WEIGHTS(I,J)*XNN(J)
C
  48     CONTINUE
C
         SUMHU = SUMHU + HUWEIGHTS(I)*DTANH(SUMVARS + BIAS(I))
C
  38  CONTINUE
C
      SUMHU = SUMHU + HUBIAS
C
      EVGVOL = (SUMHU +0.5) * MINMAX(2,9) + MINMAX(1,9)
C
      EVGVOL = EVGVOL * RHO
C
      RETURN
C
   7  PRINT *, 'READ has arrived to the end of file ', WEINAME, 
     &     ' please add some carriage feeds to the end of that file'
      RETURN
      END
C
C************************************************************************
C************************************************************************
C************************************************************************
C **********************************************************************
C                 by HKDH Bhadeshia
      SUBROUTINE MAP_STEEL_OMEGA(C,W,XBAR,T10,T20,J)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      IMPLICIT NONE
      DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER IU,J
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=C(1)/12.0115D+00
      C(2)=C(2)/28.09D+00
      C(3)=C(3)/54.94D+00
      C(4)=C(4)/58.71D+00
      C(5)=C(5)/95.94D+00
      C(6)=C(6)/52.0D+00
      C(7)=C(7)/50.94D+00
2     B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 IU=2,7
      Y(IU)=C(IU)/C(8)
107   CONTINUE
      DO 106 IU=1,8
      C(IU)=C(IU)/B1
106   CONTINUE
      XBAR=C(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
C
C  ...Polynomials representing carbon-carbon interaction energy (J/mol)
C    in austenite as a function of the molecular fraction of individual
C    solutes.
C
      P(2)=-2.4233D+07+6.9547D+07*C(2)
      P(2)=+3.864D+06+P(2)*C(2)
      P(2)=+45802.87+(-280061.63+P(2)*C(2))*C(2) 
      P(2)=2013.0341+(763.8167+P(2)*C(2))*C(2)
      P(3)=2.0119D+06+(3.1716D+07-1.3885D+08*C(3))*C(3)
      P(3)=+6287.52+(-21647.96+P(3)*C(3))*C(3)
      P(3)=2012.067+(-1764.095+P(3)*C(3))*C(3)
      P(4)=-2.4968D+07+(1.8838D+08-5.5531D+08*C(4))*C(4)
      P(4)=-54915.32+(1.6216D+06+P(4)*C(4))*C(4)
      P(4)=2006.8017+(2330.2424+P(4)*C(4))*C(4)
      P(5)=-1.3306D+07+(8.411D+07-2.0826D+08*C(5))*C(5)
      P(5)=-37906.61+(1.0328D+06+P(5)*C(5))*C(5)
      P(5)=2006.834+(-2997.314+P(5)*C(5))*C(5)
      P(6)=+8.5676D+06+(-6.7482D+07+2.0837D+08*C(6))*C(6)
      P(6)=+33657.8+(-566827.83+P(6)*C(6))*C(6)
      P(6)=2012.367+(-9224.2655+P(6)*C(6))*C(6)
      P(7)=+5411.7566
     &+(250118.1085-4.1676D+06*C(7))*C(7)
      P(7)=2011.9996+(-6247.9118+P(7)*C(7))*C(7)
      DO 108 IU=2,7
      B3=B3+P(IU)*Y(IU)
      B2=B2+Y(IU)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   CONTINUE
      RETURN
      END
C
C***********************************************************************
C***********************************************************************
C
CC H. K. D. H. Bhadeshia, Diffusivity of carbon in austenite
C Department of Materials Science and Metallurgy
C Pembroke Street, Cambridge CB2 3QZ, U. K.
C 31 March 1987
C
C Program for the calculation of the diffusivity of carbon in austenite
C as a function of carbon concentration and temperature. Also accounts for
C the infulence of C, Mn, Si, Ni, Mo, Cr and V on the activity of carbon
C in austenite, and therefore on the diffusivity of carbon.
C
C WDIFF is the carbon--carbon interaction energy in austenite, not
C allowing for the presence of Mo, Cr or V
C
C diffusion coefficient in cm**/s
C
      SUBROUTINE MAP_STEEL_DIFFUS(DIF,X,T,W,R,BOLTZ,HH)
      IMPLICIT NONE
      DOUBLE PRECISION DIF,X,T,W,PSI,THETA,ACTIV,DACTIV,DASH,SIGMA,A5
      DOUBLE PRECISION BOLTZ,HH,R,MAP_STEEL_DCG,MAP_STEEL_CG
      INTEGER IZ
         IZ=12
         A5=1.0D+00
C     DIF=DIFFUSIVITY OF CARBON IN AUSTENITE
C     IZ=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      DASH=(BOLTZ*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
      THETA=X/(A5-X)
      ACTIV=MAP_STEEL_CG(X,T,W,R)
      ACTIV=DEXP(ACTIV)
      DACTIV=MAP_STEEL_DCG(X,T,W,R)
      DACTIV=DACTIV*ACTIV
      DACTIV=DACTIV*A5/((A5+THETA)**2)
      SIGMA=A5-DEXP((-(W))/(R*T))
      PSI=ACTIV*(A5+IZ*((A5+THETA)/(A5-(A5+IZ/2)*THETA+(IZ/2)*(A5+IZ/2)*
     &(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      DIF=DASH*PSI
      RETURN
      END
C
C
C
C**********************************************************************
C**********************************************************************
C                by HKDH Bhadeshia
C
C CG CG DOUBLE PRECISION FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
      DOUBLE PRECISION FUNCTION MAP_STEEL_CG(X,T,W,R)
      IMPLICIT NONE
      DOUBLE PRECISION AJ,DG,EG,T,R,W,X
      AJ=1.0-DEXP(-W/(R*T))
         IF(X .LE. 1.0D-10) THEN
           MAP_STEEL_CG=DLOG(1.0D-10)
         ELSE
           DG=DSQRT(1.0-2.0*(1.0+2.0*AJ)*X+(1.0+8.0*AJ)*X*X)
           EG=5.0*DLOG((1.0-2.0*X)/X)+6.0*W/(R*T)+
     &     (38575.0-13.48*T)/(R*T)
           MAP_STEEL_CG=EG+6.0D+00*DLOG((DG-1+3*X)/(DG+1-3*X))
         ENDIF
      RETURN
      END
C
C*********************************************************************
C*********************************************************************
C                by HKDH Bhadeshia
C
C
C DCG DCG
C FUNCTION GIVING DIFFERENTIAL OF LN(ACTIVITY) OF CARBON IN AUSTENITE, LFG
C DIFFERENTIAL IS WITH RESPECT TO X
      DOUBLE PRECISION FUNCTION MAP_STEEL_DCG(X,T,W,R)
      IMPLICIT NONE
      DOUBLE PRECISION AJ,DG,DDG,X,T,W,R
      AJ=1.0-DEXP(-W/(R*T))
      DG=DSQRT(1.0-2.0*(1.0+2.0*AJ)*X+(1.0+8.0*AJ)*X*X)
      DDG=(0.5/DG)*(-2.0-4.0*AJ+2.0*X+16.0*AJ*X)
C***
         IF(X .LE. 1.0D-10) THEN
           X=1.0D-10
         ENDIF
C***
      MAP_STEEL_DCG=-((10.0/(1.0-2.0*X))+(5.0/X))+6.0*((DDG+3.0)
     &/(DG-1.0+3.0*X)-(DDG-3.0)/(DG+1.0-3.0*X))
      RETURN
      END
C
C********************************************************************
C********************************************************************
C
C
C
