 C************************************************************************
C
      DOUBLE PRECISION FUNCTION EVGVOL(TK, COMP)
C
C   ...  Calculates the difference in gibbs energy between ferrite&
C        cementite (taking into account composition) and austenite per
C        unit volume using a neural network which weights have been 
C        determined from data calculated using MTDATA and ...
C
C             ...Daniel Gaude-Fugarolas--March 2001 ...
C
      DOUBLE PRECISION TK, COMP(1:8)
      DOUBLE PRECISION  RHO
      DOUBLE PRECISION TEMP
      INTEGER CTRL, MAXVARS, MAXHU, VARS, HU, INW, INM, I, J, K
      PARAMETER (MAXHU=20)
      PARAMETER (MAXVARS=20)
      PARAMETER (INW=3)
      PARAMETER (INM=4)
      DOUBLE PRECISION BIAS(MAXHU), WEIGHTS(MAXHU, MAXVARS), 
     &         HUBIAS, HUWEIGHTS(MAXHU),MINMAX(2,MAXVARS+1)
      DOUBLE PRECISION SUMHU, SUMVARS, XNN(MAXVARS)
      CHARACTER*12 WEINAME, FMINMAX
      DATA  RHO / 7800.0 /
C
      SAVE  BIAS, WEIGHTS, CTRL, XNN, MINMAX,HUBIAS, HUWEIGHTS
C
      SUMHU = 0.0
C
      IF(CTRL.NE.13) THEN
C*         WRITE(*,*)' .Information about the neural network.'
C*         WRITE(*,*)'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
C*         WRITE(*,*)'Input weight file name'
C*         READ (*,*) WEINAME
C*         WRITE(*,*)'Input MINIMAX file name'
C*         READ (*,*) FMINMAX
C*         WRITE(*,*)'Input number of variables'
C*         READ (*,*) VARS
C*         WRITE(*,*)'Input number of hidden units'
C*         READ (*,*) HU
        hu =13
        vars =8
        WEINAME = 'weightfile'
        FMINMAX= 'minmaxfile'
C
         OPEN(UNIT=INW, FILE=WEINAME, STATUS='OLD')
C
         DO 67 I=1, HU
            READ(INW,*, END=7) BIAS(I)
            DO 77 J=1, VARS
               READ(INW,*, END=7) WEIGHTS(I, J)
  77        CONTINUE
  67     CONTINUE
C
         READ(INW,*, END=7) HUBIAS
         DO 97 J=1, HU
            READ(INW,*, END=7) HUWEIGHTS(J)
  97     CONTINUE
C 
         CLOSE(INW)
C
         OPEN(UNIT=INM, FILE=FMINMAX, STATUS='OLD')
C        
         DO 87 K=1, VARS+1
            READ(INM,*) MINMAX(1,K),MINMAX(2,K)
C
C  ... The difference is stored instead of the maximum ...
C
            MINMAX(2,K) = MINMAX(2,K) - MINMAX(1,K)
C
  87     CONTINUE
C
         CLOSE(INM)
C
C  ... And the vector of compositions ...
C
C  ... Fe  ...
C
         TEMP = 100.0
C 
         DO 32 I=1, 6
C
           TEMP = TEMP - COMP(I)
C 
  32     CONTINUE 
C
          XNN(1) = (((TEMP/100.0) - MINMAX(1, 1)) / 
     &          MINMAX(2,1)) - 0.5 
C
C       
         DO 37 J=2, 7
              XNN(J) = (((COMP(J-1)/100.0) - MINMAX(1, J)) / 
     &            MINMAX(2,J)) - 0.5          
  37     CONTINUE 

C
C  ...temperature is left outside the startup loop ...
C
         CTRL = 13
      ENDIF
C
C  ... Once the files have been read, they'll be kept for the rest of
C       the use of the program.
C
C  ... Now the calculations of free energy can be done using the NN ...
C
C  ...  Temperature  ...
C
      XNN(8) = ((TK - MINMAX(1, 8)) / MINMAX(2,8)) - 0.5
C
C  ...  Start of the calculations  ...
C
      SUMHU = 0.0
C
      DO 38 I=1, HU  
C
         SUMVARS = 0.0
C
         DO 48 J=1, VARS
C
            SUMVARS = SUMVARS + WEIGHTS(I,J)*XNN(J)
C
  48     CONTINUE
C
         SUMHU = SUMHU + HUWEIGHTS(I)*DTANH(SUMVARS + BIAS(I))
C
  38  CONTINUE
C
      SUMHU = SUMHU + HUBIAS
C
      EVGVOL = (SUMHU +0.5) * MINMAX(2,9) + MINMAX(1,9)
C
      EVGVOL = EVGVOL * RHO
      print *, 'Fe', temp,(comp(I),I=1, 7)
C
      RETURN
C
   7  PRINT *, 'READ has arrived to the end of file ', WEINAME, 
     &     ' please add some carriage feeds to the end of that file'
      RETURN
      END
C
C************************************************************************
